# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Jocelyn Jaubert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from datetime import datetime, date
from decimal import Decimal
from dateutil.relativedelta import relativedelta
import re

from weboob.browser import LoginBrowser, URL, need_login, StatesMixin
from weboob.exceptions import BrowserIncorrectPassword, BrowserUnavailable, ActionNeeded
from weboob.capabilities.bank import Account, TransferBankError
from weboob.capabilities.base import find_object, NotAvailable
from weboob.browser.exceptions import BrowserHTTPNotFound
from weboob.capabilities.profile import ProfileMissing

from .pages.accounts_list import (
    AccountsMainPage, AccountDetailsPage, AccountsPage, LoansPage, ComingPage, HistoryPage,
    CardListPage, CardHistoryPage,
    AdvisorPage, HTMLProfilePage, XMLProfilePage,
    MarketPage,
    LifeInsurance, LifeInsuranceHistory, LifeInsuranceInvest, LifeInsuranceInvest2,
    UnavailableServicePage,
)
from .pages.transfer import AddRecipientPage, RecipientJson, TransferJson, SignTransferPage
from .pages.login import MainPage, LoginPage, BadLoginPage, ReinitPasswordPage, ActionNeededPage, ErrorPage
from .pages.subscription import BankStatementPage


__all__ = ['SocieteGenerale']


class SocieteGenerale(LoginBrowser, StatesMixin):
    BASEURL = 'https://particuliers.societegenerale.fr'
    STATE_DURATION = 5

    # Bank
    accounts_main_page = URL('/restitution/cns_listeprestation.html',
                             '/com/icd-web/cbo/index.html', AccountsMainPage)
    account_details_page = URL('/restitution/cns_detailPrestation.html', AccountDetailsPage)
    accounts = URL('/icd/cbo/data/liste-prestations-navigation-authsec.json', AccountsPage)
    coming = URL('/restitution/cns_listeEncours.xml', ComingPage)
    history = URL('/icd/cbo/data/liste-operations-authsec.json', HistoryPage)
    cards_list = URL('/restitution/cns_listeCartesDd.html',
                     '/restitution/cns_detailCARTE_\w{3}.html', CardListPage)
    card_history = URL('/restitution/cns_listeReleveCarteDd.xml', CardHistoryPage)
    loans = URL(r'/abm/restit/listeRestitutionPretsNET.json\?a100_isPretConso=(?P<conso>\w+)', LoansPage)

    # Recipient
    add_recipient = URL(r'/personnalisation/per_cptBen_ajouterFrBic.html',
                        r'/lgn/url.html', AddRecipientPage)
    json_recipient = URL(r'/sec/getsigninfo.json',
                         r'/sec/csa/send.json',
                         r'/sec/oob_sendoob.json',
                         r'/sec/oob_polling.json', RecipientJson)
    # Transfer
    json_transfer = URL(r'/icd/vupri/data/vupri-liste-comptes.json\?an200_isBack=false',
                        r'/icd/vupri/data/vupri-check.json',
                        r'/lgn/url.html', TransferJson)
    sign_transfer = URL(r'/icd/vupri/data/vupri-generate-token.json', SignTransferPage)
    confirm_transfer = URL(r'/icd/vupri/data/vupri-save.json', TransferJson)

    # Wealth
    market = URL('/brs/cct/comti20.html', MarketPage)
    life_insurance = URL('/asv/asvcns10.html', '/asv/AVI/asvcns10a.html', '/brs/fisc/fisca10a.html', LifeInsurance)
    life_insurance_invest = URL('/asv/AVI/asvcns20a.html', LifeInsuranceInvest)
    life_insurance_invest_2 = URL('/asv/PRV/asvcns10priv.html', LifeInsuranceInvest2)
    life_insurance_history = URL('/asv/AVI/asvcns2(?P<n>[0-9])c.html', LifeInsuranceHistory)

    # Profile
    advisor = URL('/icd/pon/data/get-contacts.xml', AdvisorPage)
    html_profile_page = URL(r'/com/dcr-web/dcr/dcr-coordonnees.html', HTMLProfilePage)
    xml_profile_page = URL(r'/gms/gmsRestituerAdresseNotificationServlet.xml', XMLProfilePage)

    # Document
    bank_statement = URL(r'/restitution/rce_derniers_releves.html', BankStatementPage)
    bank_statement_search = URL(r'/restitution/rce_recherche.html\?noRedirect=1',
                                r'/restitution/rce_recherche_resultat.html', BankStatementPage)

    bad_login = URL('\/acces/authlgn.html', '/error403.html', BadLoginPage)
    reinit = URL('/acces/changecodeobligatoire.html', ReinitPasswordPage)
    action_needed = URL('/com/icd-web/forms/cct-index.html',
                        '/com/icd-web/gdpr/gdpr-recueil-consentements.html',
                        '/com/icd-web/forms/kyc-index.html',
                        ActionNeededPage)
    unavailable_service_page = URL(r'/com/service-indisponible.html', UnavailableServicePage)
    error = URL('https://static.societegenerale.fr/pri/erreur.html', ErrorPage)
    login = URL('/sec/vk', LoginPage)
    main_page = URL('https://particuliers.societegenerale.fr', MainPage)

    context = None
    dup = None
    id_transaction = None

    __states__ = ('context', 'dup', 'id_transaction')

    def load_state(self, state):
        if state.get('dup') is not None and state.get('context') is not None:
            super(SocieteGenerale, self).load_state(state)

    def do_login(self):
        if not self.password.isdigit() or len(self.password) not in (6, 7):
            raise BrowserIncorrectPassword()
        if not self.username.isdigit() or len(self.username) < 8:
            raise BrowserIncorrectPassword()
        self.username = self.username[:8]

        self.main_page.go()
        try:
            self.page.login(self.username, self.password)
        except BrowserHTTPNotFound:
            raise BrowserIncorrectPassword()

        assert self.login.is_here()
        reason, action = self.page.get_error()
        if reason == 'echec_authent':
            raise BrowserIncorrectPassword()

    def iter_cards(self, account):
        for el in account._cards:
            if el['carteDebitDiffere']:
                card = Account()
                card.id = card.number = el['numeroCompteFormate'].replace(' ', '')
                card.label = el['labelToDisplay']
                card.coming = Decimal(str(el['montantProchaineEcheance']))
                card.type = Account.TYPE_CARD
                card.currency = account.currency
                card._internal_id = el['idTechnique']
                card._prestation_id = el['id']
                yield card

    @need_login
    def get_accounts_list(self):
        self.accounts_main_page.go()

        if self.page.is_old_website():
            # go on new_website
            self.location(self.absurl('/com/icd-web/cbo/index.html'))

        # get account iban on transfer page
        self.json_transfer.go()
        account_ibans = self.page.get_account_ibans_dict()

        # get account coming on coming page, coming amount is not available yet on account page
        self.coming.go()
        account_comings = self.page.get_account_comings()

        self.accounts.go()
        for account in self.page.iter_accounts():
            for card in self.iter_cards(account):
                card.parent = account
                yield card

            if account._prestation_id in account_ibans:
                account.iban = account_ibans[account._prestation_id]

            if account._prestation_id in account_comings:
                account.coming = account_comings[account._prestation_id]

            if account.type == account.TYPE_LOAN:
                self.loans.go(conso=(account._loan_type == 'PR_CONSO'))
                account = self.page.get_loan_account(account)

            yield account

    @need_login
    def iter_card_transaction(self, account):
        # TODO
        raise BrowserUnavailable()
        self.account_details_page.go(params={'idprest': account._prestation_id})
        self.location(self.absurl(self.page.get_card_history_link(account)))

        if self.page.get_card_transactions_link():
            self.location(self.absurl(self.page.get_card_transactions_link()))

            year = date.today().year
            rdate = None
            for tr in self.page.iter_card_history():

                # search for the first card summary
                if tr.date is NotAvailable:
                    tr.type = tr.TYPE_CARD_SUMMARY
                    d = re.search(r'(\d{2})\/(\d{2})', tr.label)
                    if d:
                        dd, mm = int(d.group(1)), int(d.group(2))
                        tr.date = rdate = date(year, mm, dd)
                year = tr.date.year

                # if card summary is found, yield transaction with the right rdate
                if rdate:
                    tr.rdate = rdate
                    yield tr

    @need_login
    def iter_history(self, account):
        # TODO: check matching
        raise BrowserUnavailable()

        if account.type in (account.TYPE_LOAN, account.TYPE_MARKET, ):
            return

        if account.type == account.TYPE_CARD:
            for tr in self.iter_card_transaction(account):
                yield tr
            return

        self.history.go(params={'b64e200_prestationIdTechnique': account._internal_id})

        iter_transactions = self.page.iter_history
        if account.type == Account.TYPE_PEA:
            iter_transactions = self.page.iter_pea_history

        for transaction in iter_transactions():
            yield transaction

    @need_login
    def iter_coming(self, account):
        # TODO: check matching
        raise BrowserUnavailable()
        if account.type in (account.TYPE_LOAN, account.TYPE_MARKET ):
            return

        if account.type == account.TYPE_CARD:
            # TODO
            return

        self.history.go(params={'b64e200_prestationIdTechnique': account._internal_id})
        for transaction in self.page.iter_coming():
            yield transaction

    @need_login
    def iter_investment(self, account):
        # TODO
        raise BrowserUnavailable()
        if account.type not in (Account.TYPE_MARKET, Account.TYPE_LIFE_INSURANCE, Account.TYPE_PEA):
            self.logger.debug('This account is not supported')
            return []

        raise BrowserUnavailable()

        if account.type in (Account.TYPE_PEA, Account.TYPE_MARKET):
            self.account_details_page.go(params={'idprest': account._prestation_id})
            for invest in self.page.iter_investment():
                # TODO
                pass

        if account.type == Account.TYPE_LIFE_INSURANCE:
            # TODO
            pass

    @need_login
    def iter_recipients(self, account):
        try:
            self.json_transfer.go()
        except TransferBankError:
            return []
        if not self.page.is_able_to_transfer(account):
            return []
        return self.page.iter_recipients(account_id=account.id)

    @need_login
    def init_transfer(self, account, recipient, transfer):
        self.json_transfer.go()

        first_transfer_date = self.page.get_first_available_transfer_date()
        if transfer.exec_date and transfer.exec_date < first_transfer_date:
            transfer.exec_date = first_transfer_date

        self.page.init_transfer(account, recipient, transfer)
        return self.page.handle_response(recipient)

    @need_login
    def execute_transfer(self, transfer):
        assert transfer.id, 'Transfer token is missing'
        data = {
            'b64e200_idVirement': transfer.id
        }
        # get token and virtual keyboard
        self.sign_transfer.go(params=data)

        data.update(self.page.get_confirm_transfer_data(self.password))
        # execute transfer
        headers = {'Referer': self.absurl('/com/icd-web/vupri/virement.html')}
        self.confirm_transfer.go(data=data, headers=headers)

        assert self.page.is_transfer_validated(), 'Something went wrong, transfer is not executed'
        return transfer

    def end_sms_recipient(self, recipient, **params):
        data = [('context', self.context), ('context', self.context), ('dup', self.dup), ('code', params['code']), ('csa_op', 'sign')]
        self.add_recipient.go(data=data, headers={'Referer': 'https://particuliers.secure.societegenerale.fr/lgn/url.html'})
        return self.page.get_recipient_object(recipient)

    def end_oob_recipient(self, recipient, **params):
        r = self.open('https://particuliers.secure.societegenerale.fr/sec/oob_polling.json', data={'n10_id_transaction': self.id_transaction})
        assert r.page.doc['donnees']['transaction_status'] in ('available', 'in_progress'), \
            'transaction_status is %s' % r.page.doc['donnees']['transaction_status']

        if r.page.doc['donnees']['transaction_status'] == 'in_progress':
            raise ActionNeeded('Veuillez valider le bénéficiaire sur votre application bancaire.')

        data = [
            ('context', self.context),
            ('b64_jeton_transaction', self.context),
            ('dup', self.dup),
            ('n10_id_transaction', self.id_transaction),
            ('oob_op', 'sign')
        ]
        add_recipient_url = self.absurl('/lgn/url.html', base=True)
        self.location(
            add_recipient_url,
            data=data,
            headers={'Referer': add_recipient_url}
        )
        return self.page.get_recipient_object(recipient)

    @need_login
    def new_recipient(self, recipient, **params):
        if 'code' in params:
            return self.end_sms_recipient(recipient, **params)
        if 'pass' in params:
            return self.end_oob_recipient(recipient, **params)
        self.add_recipient.go()
        self.page.post_iban(recipient)
        self.page.post_label(recipient)
        self.page.double_auth(recipient)

    @need_login
    def get_advisor(self):
        return self.advisor.go().get_advisor()

    @need_login
    def get_profile(self):
        self.html_profile_page.go()
        profile = self.page.get_profile()
        self.xml_profile_page.go()
        profile.email = self.page.get_email()
        return profile

    @need_login
    def iter_subscription(self):
        try:
            profile = self.get_profile()
            subscriber = profile.name
        except ProfileMissing:
            subscriber = NotAvailable

        # subscriptions which have statements are present on the last statement page
        self.bank_statement.go()
        subscriptions_list = list(self.page.iter_subscription())

        # this way the no statement accounts are excluded
        # and the one keeped have all the data and parameters needed
        self.bank_statement_search.go()
        for sub in self.page.iter_searchable_subscription(subscriber=subscriber):
            found_sub = find_object(subscriptions_list, id=sub.id)
            if found_sub:
                yield sub

    @need_login
    def iter_documents(self, subscribtion):
        end_date = datetime.today()

        # 5 years since it goes with a 2 months step
        security_limit = 30
        i = 0
        while i < security_limit:
            self.bank_statement_search.go()
            self.page.post_form(subscribtion, end_date)

            # No more documents
            if self.page.has_error_msg():
                break

            for d in self.page.iter_documents(subscribtion):
                yield d

            # 3 months step because the documents list is inclusive
            # from the 08 to the 06, the 06 statement is included
            end_date = end_date - relativedelta(months=+3)
            i += 1
